@@ -13,9 +13,8 @@
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA  02111-1307  USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,11 +35,14 @@
 #include <string.h>
 #endif
 
+#include <gdk/gdkkeysyms.h>
+
 #include <terminal/terminal-app.h>
 #include <terminal/terminal-config.h>
 #include <terminal/terminal-preferences.h>
 #include <terminal/terminal-private.h>
 #include <terminal/terminal-window.h>
+#include <terminal/terminal-window-dropdown.h>
 
 #define ACCEL_MAP_PATH "xfce4/terminal/accels.scm"
 
@@ -51,12 +53,6 @@ static void               terminal_app_update_accels            (TerminalApp
 static void               terminal_app_update_mnemonics         (TerminalApp        *app);
 static gboolean           terminal_app_accel_map_load           (gpointer            user_data);
 static gboolean           terminal_app_accel_map_save           (gpointer            user_data);
-static GtkWidget         *terminal_app_create_window            (TerminalApp        *app,
-                                                                 const gchar        *role,
-                                                                 gboolean            fullscreen,
-                                                                 TerminalVisibility  menubar,
-                                                                 TerminalVisibility  borders,
-                                                                 TerminalVisibility  toolbar);
 static void               terminal_app_new_window               (TerminalWindow     *window,
                                                                  const gchar        *working_directory,
                                                                  TerminalApp        *app);
@@ -259,6 +255,8 @@ terminal_app_accel_map_load (gpointer user_data)
 {
   TerminalApp *app = TERMINAL_APP (user_data);
   gchar       *path;
+  gchar        name[50];
+  guint        i;
 
   app->accel_map_load_id = 0;
 
@@ -269,6 +267,15 @@ terminal_app_accel_map_load (gpointer user_data)
       gtk_accel_map_load (path);
       g_free (path);
     }
+  else
+    {
+      /* create default Alt+N accelerators */
+      for (i = 1; i < 10; i++)
+        {
+          g_snprintf (name, sizeof (name), "<Actions>/terminal-window/goto-tab-%d", i);
+          gtk_accel_map_change_entry (name, GDK_0 + i, GDK_MOD1_MASK, FALSE);
+        }
+    }
 
   /* watch for changes */
   app->accel_map = gtk_accel_map_get ();
@@ -280,6 +287,29 @@ terminal_app_accel_map_load (gpointer user_data)
 
 
 
+static void
+terminal_app_take_window (TerminalApp *app,
+                          GtkWindow   *window)
+{
+  GtkWindowGroup *group;
+
+  terminal_return_if_fail (GTK_IS_WINDOW (window));
+
+  group = gtk_window_group_new ();
+  gtk_window_group_add_window (group, window);
+  g_object_weak_ref (G_OBJECT (window), (GWeakNotify) g_object_unref, group);
+
+  g_signal_connect (G_OBJECT (window), "destroy",
+                    G_CALLBACK (terminal_app_window_destroyed), app);
+  g_signal_connect (G_OBJECT (window), "new-window",
+                    G_CALLBACK (terminal_app_new_window), app);
+  g_signal_connect (G_OBJECT (window), "new-window-with-screen",
+                    G_CALLBACK (terminal_app_new_window_with_terminal), app);
+  app->windows = g_slist_prepend (app->windows, window);
+}
+
+
+
 static GtkWidget*
 terminal_app_create_window (TerminalApp       *app,
                             const gchar       *role,
@@ -290,7 +320,6 @@ terminal_app_create_window (TerminalApp       *app,
 {
   GtkWidget *window;
   gchar     *new_role = NULL;
-  GtkWindowGroup *group;
 
   if (role == NULL)
     {
@@ -300,21 +329,28 @@ terminal_app_create_window (TerminalApp       *app,
     }
 
   window = terminal_window_new (role, fullscreen, menubar, borders, toolbar);
-
-  group = gtk_window_group_new ();
-  gtk_window_group_add_window (group, GTK_WINDOW (window));
-  g_object_weak_ref (G_OBJECT (window), (GWeakNotify) g_object_unref, group);
-
-  g_signal_connect (G_OBJECT (window), "destroy",
-                    G_CALLBACK (terminal_app_window_destroyed), app);
-  g_signal_connect (G_OBJECT (window), "new-window",
-                    G_CALLBACK (terminal_app_new_window), app);
-  g_signal_connect (G_OBJECT (window), "new-window-with-screen",
-                    G_CALLBACK (terminal_app_new_window_with_terminal), app);
-  app->windows = g_slist_prepend (app->windows, window);
-
   g_free (new_role);
 
+  terminal_app_take_window (app, GTK_WINDOW (window));
+
+  return window;
+}
+
+
+
+static GtkWidget*
+terminal_app_create_drop_down (TerminalApp        *app,
+                               const gchar        *role,
+                               gboolean            fullscreen,
+                               TerminalVisibility  menubar,
+                               TerminalVisibility  toolbar)
+{
+  GtkWidget *window;
+
+  window = terminal_window_dropdown_new (role, fullscreen, menubar, toolbar);
+
+  terminal_app_take_window (app, GTK_WINDOW (window));
+
   return window;
 }
 
@@ -428,9 +464,13 @@ terminal_app_save_yourself (XfceSMClient *client,
   gint                  argc;
   gint                  n;
 
-  for (lp = app->windows; lp != NULL; lp = lp->next)
+  for (lp = app->windows, n = 0; lp != NULL; lp = lp->next)
     {
-      if (lp != app->windows)
+      /* don't session save dropdown windows */
+      if (TERMINAL_IS_WINDOW_DROPDOWN (lp->data))
+        continue;
+
+      if (n++ != 0)
         result = g_slist_append (result, g_strdup ("--window"));
       result = g_slist_concat (result, terminal_window_get_restart_command (lp->data));
     }
@@ -544,8 +584,40 @@ terminal_app_open_window (TerminalApp        *app,
   terminal_return_if_fail (TERMINAL_IS_APP (app));
   terminal_return_if_fail (attr != NULL);
 
-  if (attr->reuse_last_window
-      && app->windows != NULL)
+  if (attr->drop_down)
+    {
+      /* look for an exising drop-down window */
+      for (lp = app->windows; lp != NULL; lp = lp->next)
+        if (TERMINAL_IS_WINDOW_DROPDOWN (lp->data))
+          break;
+
+      if (lp != NULL)
+        {
+          if (G_UNLIKELY (attr->reuse_last_window))
+            {
+              /* use the drop-down window to insert the tab */
+              window = lp->data;
+              reuse_window = TRUE;
+            }
+          else
+            {
+              /* toggle state of visible window */
+              terminal_window_dropdown_toggle (lp->data, attr->startup_id, FALSE);
+              return;
+            }
+        }
+      else
+        {
+          /* create new drop-down window */
+          window = terminal_app_create_drop_down (app,
+                                                  attr->role,
+                                                  attr->fullscreen,
+                                                  attr->menubar,
+                                                  attr->toolbar);
+        }
+    }
+  else if (attr->reuse_last_window
+           && app->windows != NULL)
     {
       /* open the tabs in an existing window */
       window = app->windows->data;
@@ -561,11 +633,13 @@ terminal_app_open_window (TerminalApp        *app,
                                            attr->borders,
                                            attr->toolbar);
 
-      if (attr->startup_id != NULL)
-        gtk_window_set_startup_id (GTK_WINDOW (window), attr->startup_id);
+      /* apply normal window properties */
       if (attr->maximize)
         gtk_window_maximize (GTK_WINDOW (window));
 
+      if (attr->startup_id != NULL)
+        gtk_window_set_startup_id (GTK_WINDOW (window), attr->startup_id);
+
       if (attr->icon != NULL)
         {
           if (g_path_is_absolute (attr->icon))
@@ -582,6 +656,7 @@ terminal_app_open_window (TerminalApp        *app,
         }
     }
 
+  /* add the tabs */
   for (lp = attr->tabs; lp != NULL; lp = lp->next)
     {
       terminal = g_object_new (TERMINAL_TYPE_SCREEN, NULL);
@@ -600,27 +675,33 @@ terminal_app_open_window (TerminalApp        *app,
       terminal_screen_launch_child (TERMINAL_SCREEN (terminal));
     }
 
-  /* don't apply other attributes to teh window when reusing */
-  if (reuse_window)
-    return;
+  if (!attr->drop_down)
+    {
+      /* don't apply other attributes to the window when reusing */
+      if (reuse_window)
+        return;
 
-  /* set the window geometry, this can only be set after one of the tabs
-   * has been added, because vte is the geometry widget, so atleast one
-   * call should have been made to terminal_screen_set_window_geometry_hints */
-  if (G_LIKELY (attr->geometry == NULL))
-    g_object_get (G_OBJECT (app->preferences), "misc-default-geometry", &geometry, NULL);
-  else
-    geometry = g_strdup (attr->geometry);
+      /* set the window geometry, this can only be set after one of the tabs
+       * has been added, because vte is the geometry widget, so atleast one
+       * call should have been made to terminal_screen_set_window_geometry_hints */
+      if (G_LIKELY (attr->geometry == NULL))
+        g_object_get (G_OBJECT (app->preferences), "misc-default-geometry", &geometry, NULL);
+      else
+        geometry = g_strdup (attr->geometry);
 
-  /* try to apply the geometry to the window */
-  if (!gtk_window_parse_geometry (GTK_WINDOW (window), geometry))
-    g_printerr (_("Invalid geometry string \"%s\"\n"), geometry);
+      /* try to apply the geometry to the window */
+      if (!gtk_window_parse_geometry (GTK_WINDOW (window), geometry))
+        g_printerr (_("Invalid geometry string \"%s\"\n"), geometry);
 
-  /* cleanup */
-  g_free (geometry);
+      /* cleanup */
+      g_free (geometry);
+    }
 
   /* show the window */
-  gtk_widget_show (window);
+  if (attr->drop_down)
+    terminal_window_dropdown_toggle (TERMINAL_WINDOW_DROPDOWN (window), attr->startup_id, reuse_window);
+  else
+    gtk_widget_show (window);
 }
 
 
